import { Request } from '@libeilong/request';

class ApiError extends Error {
  constructor(message, code) {
    super(`${code ? `【${code}】` : ''} ${message}`);
    this.code = code;
  }

}

let time, ticket;
class TicketHandler {
  /**
   * 获取 jsapi-ticket
   *
   * 该接口成功获取 ticket 之后会调用 setTicketHandler 传入的 setTicket
   *
   */
  async fetchJsapiTicket() {
    if (this.last) return this.last;
    this.last = new Promise(async (ok, fail) => {
      try {
        const res = await this._fetchJsapiTicket();
        ok(res);
      } catch (err) {
        fail(err);
      } finally {
        this.last = undefined;
      }
    });
    return this.last;
  }

  async _fetchJsapiTicket() {
    const token = await this.getToken();
    const res = await this.request.get('/cgi-bin/ticket/getticket', {
      access_token: token,
      type: 'jsapi'
    });
    const {
      ticket
    } = res.isSuccess();
    await this.setTicket(ticket);
    return ticket;
  }

  async getTicket() {
    if (!ticket) {
      return await this.fetchJsapiTicket();
    }

    return ticket;
  }

  async setTicket(_ticket) {
    clearTimeout(time);
    ticket = _ticket;
    time = setTimeout(() => {
      ticket = undefined;
    }, 1000 * 60 * 90);
  }

  setJsapiTicketHandler(handlers) {
    this.getTicket = handlers.getTicket;
    this.setTicket = handlers.setTicket;
  }

}

let time$1, token;
class TokenHandler {
  /**
   * 获取 AccessToken
   *
   * 该接口成功获取 token 之后会调用 setTokenHandler 传入的 setToken
   *
   * https://developers.weixin.qq.com/doc/offiaccount/Basic_Information/Get_access_token.html
   *
   */
  async fetchToken() {
    if (this.last) return this.last;
    this.last = new Promise(async (ok, fail) => {
      try {
        const res = await this._fetchToken();
        ok(res);
      } catch (err) {
        fail(err);
      } finally {
        this.last = undefined;
      }
    });
    return this.last;
  }

  async _fetchToken() {
    const res = await this.request.get('/cgi-bin/token', {
      grant_type: 'client_credential',
      appid: this.appid,
      secret: this.appsecret
    });
    const {
      access_token
    } = res.isSuccess();
    await this.setToken(access_token);
    return access_token;
  }

  async getToken() {
    if (!token) {
      return await this.fetchToken();
    }

    return token;
  }

  async setToken(_token) {
    clearTimeout(time$1);
    token = _token;
    time$1 = setTimeout(() => {
      token = undefined;
    }, 1000 * 60 * 90);
  }

  setTokenHandler(handlers) {
    this.getToken = handlers.getToken;
    this.setToken = handlers.setToken;
  }

}

function buffer2Text(buffer) {
  const enc = new TextDecoder('utf-8');
  return enc.decode(new Uint8Array(buffer));
}
function applyMixins(derivedCtor, constructors) {
  constructors.forEach(baseCtor => {
    Object.getOwnPropertyNames(baseCtor.prototype).forEach(name => {
      Object.defineProperty(derivedCtor.prototype, name, Object.getOwnPropertyDescriptor(baseCtor.prototype, name) || Object.create(null));
    });
  });
}

class WechatBase {
  constructor(options) {
    this.endpoint = 'api.weixin.qq.com';
    this.request = new Request({
      baseURL: `https://${this.endpoint}`,
      successCondition: result => {
        return result.code === 0;
      },
      onError: res => {
        throw new ApiError(res.message, res.code);
      },
      onResponse: (result, response) => {
        let data = response.data; // 处理图片

        if (data instanceof Buffer) {
          try {
            data = JSON.parse(buffer2Text(data));
            result.code = data.errcode || 0;
            result.message = data.errmsg;
            result.data = data;
          } catch (err) {
            result.code = 0;
            result.message = 'buffer';
            result.data = Buffer.from(data);
          }
        } else {
          result.code = data.errcode || 0;
          result.message = data.errmsg;
          result.data = data;
        }
      }
    });

    this.setEndpoint = endpoint => {
      this.endpoint = endpoint;
    };

    this.setAppid = appid => {
      this.appid = appid;
    };

    this.setAppsecret = appsecret => {
      this.appsecret = appsecret;
    };

    if (!options) return;
    if (options.appid) this.appid = options.appid;
    if (options.appsecret) this.appsecret = options.appsecret;

    if (options.endpoint) {
      this.endpoint = options.endpoint;
      this.request.getInstance().defaults.baseURL = `https://${this.endpoint}`;
    }
  }

}
applyMixins(WechatBase, [TokenHandler]);

class Code2Session {
  /**
   * 本接口应在服务器端调用，详细说明参见服务端API。
   * 登录凭证校验。通过 [wx.login](https://developers.weixin.qq.com/miniprogram/dev/api/open-api/login/wx.login.html) 接口获得临时登录凭证 code 后传到开发者服务器调用此接口完成登录流程。更多使用方法详见 [小程序登录](https://developers.weixin.qq.com/miniprogram/dev/framework/open-ability/login.html)。
   * https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/login/auth.code2Session.html
   */
  async code2Session(options) {
    try {
      const res = await this.root.request.get('sns/jscode2session', { ...options,
        appid: this.root.appid,
        secret: this.root.appsecret,
        grant_type: 'authorization_code'
      });
      return res.isSuccess();
    } catch (err) {
      console.log('请求失败', err);
      throw err;
    }
  }

}

class Auth {
  constructor(root) {
    this.root = root;
  }

}
applyMixins(Auth, [Code2Session]);

class SubmitPages {
  /**
   * 小程序开发者可以通过本接口提交小程序页面url及参数信息(不要推送webview页面)，让微信可以更及时的收录到小程序的页面信息，开发者提交的页面信息将可能被用于小程序搜索结果展示。
   *
   * @example
   * ```ts
   * client.search.submitPages({
   *   pages: [
   *     { path: "pages/news/index", query: "id=11" },
   *     { path: "pages/article/index", query: "id=22" }
   *   ]
   * })
   * ```
   *
   * https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/search/search.submitPages.html
   *
   */
  async submitPages(options) {
    const token = await this.root.getToken();
    const res = await this.root.request.post('/wxa/search/wxaapi_submitpages?access_token=' + token, { ...options
    });
    return res.isSuccess();
  }

}

class Search {
  constructor(root) {
    this.root = root;
  }

}
applyMixins(Search, [SubmitPages]);

class MsgSexCheck {
  /**
   * 检查一段文本是否含有违法违规内容。
   *
   * 应用场景举例：
   *
   * 用户个人资料违规文字检测；
   *
   * 媒体新闻类用户发表文章，评论内容检测；
   *
   * 游戏类用户编辑上传的素材(如答题类小游戏用户上传的问题及答案)检测等。 频率限制：单个 appId 调用上限为 4000 次/分钟，2,000,000 次/天*
   *
   * https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/sec-check/security.msgSecCheck.html
   *
   */
  async msgSexCheck(options) {
    const token = await this.root.getToken();
    const res = await this.root.request.post('/wxa/msg_sec_check?access_token=' + token, { ...options
    });
    return res.isSuccess();
  }

}

class Security {
  constructor(root) {
    this.root = root;
  }

}
applyMixins(Security, [MsgSexCheck]);

class Send {
  /**
   * 发送订阅消息
   *
   * https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/subscribe-message/subscribeMessage.send.html
   *
   * @throws {ApiError}
   */
  async send(options) {
    const token = await this.root.getToken();
    const res = await this.root.request.post('/cgi-bin/message/subscribe/send?access_token=' + token, { ...options
    });
    return res.isSuccess();
  }

}

class SubscribeMessage {
  constructor(root) {
    this.root = root;
  }

}
applyMixins(SubscribeMessage, [Send]);

class CreateQRCode {
  /**
   * 获取小程序二维码，适用于需要的码数量较少的业务场景。通过该接口生成的小程序码，永久有效，有数量限制。
   * https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/qr-code/wxacode.createQRCode.html
   */
  async createQRCode(options) {
    const token = await this.root.getToken();
    const res = await this.root.request.post(`/cgi-bin/wxaapp/createwxaqrcode?access_token=${token}`, { ...options
    }, {
      responseType: 'arraybuffer'
    });
    return res.isSuccess();
  }

}

class GetUnlimited {
  /**
   * 获取小程序码，适用于需要的码数量极多的业务场景。通过该接口生成的小程序码，永久有效，数量暂无限制。
   *
   * 需要注意的是，该方法需要在小程序发布了正式版才可正常调用，否则会获得报错：41030 invalid page rid
   *
   * https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/qr-code/wxacode.getUnlimited.html
   */
  async getUnlimited(options) {
    const token = await this.root.getToken();
    const res = await this.root.request.post(`/wxa/getwxacodeunlimit?access_token=${token}`, { ...options
    }, {
      responseType: 'arraybuffer'
    });
    return res.isSuccess();
  }

}

class Wxacode {
  constructor(root) {
    this.root = root;
  }

}
applyMixins(Wxacode, [CreateQRCode, GetUnlimited]);

class MiniAPI extends WechatBase {
  constructor() {
    super(...arguments);
    /** 内容安全 */

    this.security = new Security(this);
    /** 小程序码 */

    this.wxacode = new Wxacode(this);
    /** 订阅消息 */

    this.subscribeMessage = new SubscribeMessage(this);
    /** 登录 */

    this.auth = new Auth(this);
    /** 小程序搜索 */

    this.search = new Search(this);
  }

}
applyMixins(MiniAPI, [TicketHandler]);

var QRCode;

(function (QRCode) {
  /** 临时 */
  QRCode[QRCode["temporary"] = 0] = "temporary";
  /** 永久 */

  QRCode[QRCode["permanent"] = 1] = "permanent";
})(QRCode || (QRCode = {}));

class CreateQRCode$1 {
  /**
   * 创建二维码
   * https://developers.weixin.qq.com/doc/offiaccount/Account_Management/Generating_a_Parametric_QR_Code.html
   * @returns
   */
  async createQRCode(options) {
    const token = await this.root.getToken();
    const res = await this.root.request.post(`/cgi-bin/qrcode/create?access_token=${token}`, {
      action_name: options.type === QRCode.temporary ? 'QR_STR_SCENE' : 'QR_LIMIT_SCENE',
      action_info: {
        scene: {
          scene_str: options.scene_str
        }
      },
      ...{
        expire_seconds: options.expire_seconds
      }
    });
    const data = res.isSuccess();
    return { ...data,
      ticket_url: 'https://mp.weixin.qq.com/cgi-bin/showqrcode?ticket=' + data.ticket
    };
  }

}

class GetAccessToken {
  /**
   * 通过用户 Code 获取网页授权 access_token
   * https://developers.weixin.qq.com/doc/offiaccount/OA_Web_Apps/Wechat_webpage_authorization.html#1
   */
  async getAccessToken(options) {
    const res = await this.root.request.get(`/sns/oauth2/access_token`, { ...options,
      appid: this.root.appid,
      secret: this.root.appsecret,
      grant_type: 'authorization_code'
    });
    if (res.data.errcode) throw Error(res.data.errcode + '：' + res.data.errmsg);
    return res.isSuccess();
  }

}

class GetUserInfo {
  /**
   * 获取用户信息
   *
   * 如果网页授权作用域为snsapi_userinfo，则此时开发者可以通过access_token和openid拉取用户信息了。
   *
   * 注意：此方法与 user.getUserInfo 不同
   * https://developers.weixin.qq.com/doc/offiaccount/OA_Web_Apps/Wechat_webpage_authorization.html#3
   */
  async getUserInfo(options) {
    options = { ...{
        lang: 'zh_CN'
      },
      ...options
    };
    const res = await this.root.request.get(`/sns/userinfo`, { ...options
    });
    if (res.data.errcode) throw Error(res.data.errcode + '：' + res.data.errmsg);
    return res.isSuccess();
  }

}

class Auth$1 {
  constructor(root) {
    this.root = root;
  }

}
applyMixins(Auth$1, [GetAccessToken, GetUserInfo]);

class Qrcode {
  constructor(root) {
    this.root = root;
  }

}
applyMixins(Qrcode, [CreateQRCode$1]);

class SendTemplate {
  /**
   * 发送模板消息
   *
   * https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Template_Message_Interface.html#5
   *
   * 注：url和miniprogram都是非必填字段，若都不传则模板无跳转；若都传，会优先跳转至小程序。开发者可根据实际需要选择其中一种跳转方式即可。当用户的微信客户端版本不支持跳小程序时，将会跳转至url。
   */
  async sendTemplate(options) {
    const token = await this.root.getToken();
    const res = await this.root.request.post('/cgi-bin/message/template/send?access_token=' + token, { ...options
    });
    return res.isSuccess();
  }

}

class Template {
  constructor(root) {
    this.root = root;
  }

}
applyMixins(Template, [SendTemplate]);

class GetFollowers {
  /**
   * 获取关注者列表
   * https://developers.weixin.qq.com/doc/offiaccount/User_Management/Getting_a_User_List.html
   * @returns
   */
  async getFollowers(options) {
    const token = await this.root.getToken();
    const res = await this.root.request.get(`/cgi-bin/user/get`, { ...options,
      access_token: token
    });
    return res.isSuccess();
  }

}

class GetUserInfo$1 {
  /**
   * 获取用户基本信息
   *
   * https://developers.weixin.qq.com/doc/offiaccount/User_Management/Get_users_basic_information_UnionID.html
   */
  async getUserInfo(options) {
    const token = await this.root.getToken();
    const res = await this.root.request.get('/cgi-bin/user/info', { ...options,
      access_token: token
    });
    return res.isSuccess();
  }

}

class UpdateRemark {
  /**
   * 更新用户备注名
   *
   * https://developers.weixin.qq.com/doc/offiaccount/User_Management/Configuring_user_notes.html
   *
   *
   */
  async updateRemark(options) {
    const token = await this.root.getToken();
    const res = await this.root.request.post(`/cgi-bin/user/info/updateremark?access_token=${token}`, { ...options
    });
    return res.isSuccess();
  }

}

class User {
  constructor(root) {
    this.root = root;
  }

}
applyMixins(User, [GetFollowers, GetUserInfo$1, UpdateRemark]);

class WebAPI extends WechatBase {
  constructor() {
    super(...arguments);
    /** 二维码 */

    this.qrcode = new Qrcode(this);
    /** 用户 */

    this.user = new User(this);
    /** 网页授权 */

    this.auth = new Auth$1(this);
    /** 模板消息 */

    this.template = new Template(this);
  }

}
applyMixins(WebAPI, [TicketHandler]);

export { ApiError, MiniAPI, QRCode, WebAPI };
//# sourceMappingURL=wechat-api-next.esm.js.map
